#-----------------------------------------------------------------------------
# Version: 1.1.0
# Compatible: SketchUp 7 (PC)
#             (other versions untested)
#-----------------------------------------------------------------------------
#
# CHANGELOG
# 1.1.0 - 07.07.2010
#		 * Different colour per tube.
#		 * Merged to one tool. Ctrl to toggle.
#
# 1.0.0 - 07.07.2010
#		 * Initial release.
#
#-----------------------------------------------------------------------------
#
# Thomas Thomassen
# thomas[at]thomthom[dot]net
#
#-----------------------------------------------------------------------------

require 'sketchup.rb'

#-----------------------------------------------------------------------------

module TT_Superglue
	
	### MENU & TOOLBARS ### --------------------------------------------------
	
	unless file_loaded?( File.basename(__FILE__) )
    root_menu = ($tt_menu) ? $tt_menu : UI.menu('Tools')
		root_menu.add_item('Superglue')   { self.select_tool(SuperGlue) }
	end
	
	
	### MAIN SCRIPT ### ------------------------------------------------------
  
    
  def self.select_tool(tool)
    Sketchup.active_model.tools.push_tool( tool.new )
  end
  
  
  class SuperGlue
    
    SUPERGLUE = 0
    SOLVENT = 1
    
    def initialize
      # Icon: http://mattahan.deviantart.com/art/Buuf-37966044?q=boost%3Apopular+buuf+icon&qo=0
      #       Paul Davey aka Mattahan. All rights reserved. 
      plugins_path = File.expand_path( File.dirname(__FILE__) )
      cursor_superglue = File.join( plugins_path, 'TT_Superglue', 'Superglue.png')
      cursor_solvent   = File.join( plugins_path, 'TT_Superglue', 'Solvent.png')
      @cursor_superglue = UI.create_cursor(cursor_superglue,  15, 0)
      @cursor_solvent   = UI.create_cursor(cursor_solvent,    15, 0)
      
      @mode = SUPERGLUE
      @colour = [255,192,0]
    end
    
    def onSetCursor
      if @mode == SUPERGLUE
        UI.set_cursor(@cursor_superglue)
      else
        UI.set_cursor(@cursor_solvent)
      end
    end
    
    def updateUI
      if @mode == SUPERGLUE
        Sketchup.status_text = 'Superglue - sweep across components to glue them to the face they lie on. Press Ctrl to switch to Solvent.'
      else
        Sketchup.status_text = 'Solvent - sweep across components to unglue them. Press Ctrl to switch to Superglue.'
      end
    end
    
    def activate
      @path = []
      @face = nil
      @bp = nil
      @ap = nil
      updateUI()
    end
    
    def deactivate(view)
      view.invalidate
    end
    
    def resume(view)
      updateUI()
    end
    
    def onKeyUp(key, repeat, flags, view)
      if key == COPY_MODIFIER_KEY
        if @mode == SUPERGLUE 
          @mode = SOLVENT
        else
          @mode = SUPERGLUE
        end
        onLButtonUp(flags, 0, 0, view)
        onSetCursor()
        updateUI()
      end
    end
    
    def onLButtonDown(flags, x, y, view)
      @flags = flags
      @path.clear
      
      operation_name = (@mode == SUPERGLUE) ? 'Superglue' : 'Solvent'
      view.model.start_operation(operation_name)
      view.invalidate
    end
    
    def onMouseMove(flags, x, y, view)
      @flags = flags
      if flags & MK_LBUTTON == MK_LBUTTON
        @path << Geom::Point3d.new(x, y, 0)
        
        ph = view.pick_helper
        ph.do_pick(x, y)
        
        @bp = ph.best_picked
        @ap = ph.all_picked
        
        # sub-class MUST implement this
        if @mode == SUPERGLUE
          onSqueezeGlue(flags, x, y, view, ph)
        else
          onSqueezeSolvent(flags, x, y, view, ph)
        end
      end
      view.invalidate
    end
    
    def onLButtonUp(flags, x, y, view)
      @flags = flags
      @path.clear
      view.model.commit_operation
      view.invalidate
    end
    
    
    def onSqueezeGlue(flags, x, y, view, ph)
      if ph.picked_face && ph.all_picked.include?( ph.picked_face )
        @face = ph.picked_face
        
        view.model.selection.clear
        view.model.selection.add( @face )
      end
      
      if @bp.nil?
        @face = nil
      end
      
      if @face && @bp.respond_to?( :glued_to )
        dfn = @bp.definition
        if dfn.behavior.is2d? &&
           @bp.transformation.zaxis.samedirection?( @face.normal ) &&
           @face.classify_point( @bp.transformation.origin ) <= 8
              @bp.glued_to = @face
        end
      end
    end
    
    
    def onSqueezeSolvent(flags, x, y, view, ph)
      if @bp && @bp.respond_to?( :glued_to ) && @bp.glued_to
        @bp.glued_to = nil
      end
    end
    
    
    def draw(view)
      #view.draw_text( [100,50,0], "onMouseMove flags: #{@flags}" )
      #view.draw_text( [100,70,0], "@face: #{@face}" )
      #view.draw_text( [100,90,0], "@bp: #{@bp}" )
      #view.draw_text( [100,110,0], "@ap: #{@ap}" )
      if @path.size > 1
        view.line_stipple = ''
        view.line_width = 5
        view.drawing_color = (@mode == SUPERGLUE) ? [255,192,0] : [128,192,255]
        view.draw2d(GL_LINE_STRIP, @path)
        #view.draw_points(@path, 20, 4, [255,0,0])
      end
    end
    
  end # module Tube

	
	### HELPER METHODS ### ---------------------------------------------------
	
	def self.start_operation(name)
		model = Sketchup.active_model
		if Sketchup.version.split('.')[0].to_i >= 7
			model.start_operation(name, true)
		else
			model.start_operation(name)
		end
	end
	
end # module

#-----------------------------------------------------------------------------
file_loaded( File.basename(__FILE__) )
#-----------------------------------------------------------------------------